require_relative "./card/request.rb"

module Seedu
  class Card

    attr_accessor :title, :description, :image_uri
    REQUIRED_FIELDS = [:title, :description, :duration, :coordinates].freeze

    def initialize(options)
      @title = options[:title]
      @description = options[:description]
      @image_uri = options[:image_uri]
      @duration = options[:duration]
      @coordinates = options[:coordinates]
      @actions = []
    end

    ##
    # Adds an action to the card.
    def add_action(action)
      if Action.constants.any? {|sym| action.class.name == "Seedu::Action::#{sym}" }
        @actions << action
      else
        raise NotASeeduAction
      end
    end

    ##
    # Returns the card as a hash suitable for conversion to JSON
    def as_json
      {
        title: @title,
        description: @description,
        image_uri: @image_uri,
        duration: @duration,
        coordinates: @coordinates,
        actions: @actions.map(&:as_json)
      }
    end

    ##
    # Returns the card as a JSON string.
    def to_json
      as_json.to_json
    end

    ##
    # Sends the card to the Seedu service.
    def send!(uri = nil)
      self.as_json.tap {|payload|
        REQUIRED_FIELDS.each {|field|
          raise IncompleteCardDetails.new("Missing field: #{field}", payload) if !payload.has_key?(field) || !payload[field]
        }
        return Request.send("#{uri || API_URI}/cards.json", payload)
      }
    end

  end
end
